@@ -5,7 +5,7 @@ import matplotlib
 import cv2 as cv
 from src import stats
 from src import ImageProcessing
-matplotlib.use('Agg')
+matplotlib.use('pdf')
 import matplotlib.pyplot as plt
 import statistics as stat
 from skimage import feature
@@ -102,6 +102,77 @@ class getStripe:
             del(hs)
         return res
 
+    def getQuantile_slow(self, coolinfo, ChrList, quantile):
+        def SplitVal(k,w,r_start,r_end):
+            print(k)
+            with np.errstate(divide='ignore', invalid='ignore'):
+                w_start = k * w
+                w_start += r_start
+                w_end = (k + 1) * w - 1
+                if w_end >= np.floor(CHROMSIZE / nbin):
+                    w_end = int(np.floor(CHROMSIZE / nbin))
+                w_end += r_start
+
+                w_start = int(w_start)
+                w_end = int(w_end)
+                r_start = int(r_start)
+                r_end = int(r_end)
+                temp = self.unbalLib[w_start:w_end, r_start:r_end][self.unbalLib[w_start:w_end, r_start:r_end] > 0]
+                return temp
+
+        res = {}
+        chrom_size = coolinfo.chromsizes
+        chrom_cum_size = np.nancumsum(chrom_size)
+        chrom_names = chrom_size.keys()
+        nbin = coolinfo.binsize
+        # Index for selected chromosomes
+        chridx = [c for c in range(len(chrom_names)) if chrom_names[c] in ChrList]
+        chridx.sort()
+        for ci in chridx:
+            CHROM = chrom_names[ci]
+            CHROMSIZE = chrom_size[ci]
+            L = int(np.ceil(CHROMSIZE / nbin))
+            if ci == 0:
+                r_start = 0
+            else:
+                r_start = chrom_cum_size[ci - 1]
+
+            r_end = chrom_cum_size[ci]
+
+            r_start = np.ceil(r_start / nbin) + 1
+            r_end = np.ceil(r_end / nbin)
+
+            hs = np.empty(0)
+
+            w = int(np.floor(25000000 / L))
+
+            N_windows = int(np.ceil(L / w))
+
+            W = Parallel(n_jobs=1)(delayed(SplitVal)(ww,w,r_start,r_end) for ww in tqdm(range(N_windows)))
+            for i in range(len(W)):
+                hs = np.concatenate((hs, W[i]))
+            qt = np.quantile(hs, quantile)
+            res[CHROM] = qt
+            del(hs)
+        return res
+
+    def getQuantile_original(self, coolinfo, ChrList, quantile):
+
+        res = {}
+        chrom_size = coolinfo.chromsizes
+        chrom_names = chrom_size.keys()
+        # Index for selected chromosomes
+        chridx = [c for c in range(len(chrom_names)) if chrom_names[c] in ChrList]
+
+        chridx.sort()
+        for ci in chridx:
+            CHROM = chrom_names[ci]
+
+            mat = self.unbalLib.fetch(CHROM)
+            qt = np.quantile(mat[mat>0], quantile)
+            res[CHROM] = qt
+            del mat
+        return res
 
     def mpmean(self):
 
@@ -163,197 +234,223 @@ class getStripe:
 
 
     def nulldist(self):
-        np.seterr(divide='ignore', invalid='ignore')
-        t_background_start = time.time()
+        with np.errstate(divide='ignore', invalid='ignore'):
+            t_background_start = time.time()
 
-        #tableft = [[0 for j in range(1000)] for i in range(400)]
-        #tabcenter = [[0 for j in range(1000)] for i in range(400)]
-        #tabright = [[0 for j in range(1000)] for i in range(400)]
+            samplesize = (self.all_chromsizes / np.sum(self.all_chromsizes)) * 1000
+            samplesize = np.uint64(samplesize)
+            dif = 1000 - np.sum(samplesize)
+            notzero = np.where(samplesize != 0)
+            chromnames2 = [self.all_chromnames[i] for i in notzero[0]]
+            samplesize[0] += dif
 
-        samplesize = (self.all_chromsizes / np.sum(self.all_chromsizes)) * 1000
-        samplesize = np.uint64(samplesize)
-        dif = 1000 - np.sum(samplesize)
-        notzero = np.where(samplesize != 0)
-        chromnames2 = [self.all_chromnames[i] for i in notzero[0]]
-        samplesize[0] += dif
+            def available_cols(chr):
+                with np.errstate(divide='ignore', invalid='ignore'):
+                    chr = str(chr)
+                    chrsize = self.chromnames2sizes[chr]
+                    itera = min(chrsize/self.resol/500, 25)
+                    itera = np.uint64(itera)
+                    unitsize = np.floor(chrsize/self.resol/itera)
+                    unitsize = np.uint64(unitsize)
+                    poolsum = 0
+                    for it in range(itera):
+                        test_region_start = np.uint64(unitsize * self.resol * it+1)
+                        test_region_end = np.uint64(unitsize * self.resol * (it+1))
+                        if test_region_start > test_region_end:
+                            a = test_region_start
+                            test_region_start = test_region_end
+                            test_region_end = a
+                        position = str(chr) + ":" + str(test_region_start) + "-" + str(test_region_end)
+                        mat = self.unbalLib.fetch(position, position)
+                        mat = nantozero(mat)
+                        #mat = np.round(mat, 1)
+                        matsum = np.sum(mat, axis=1)
+                        zeroindex = np.where(matsum == 0)
+                        poolsum += (len(matsum) - len(zeroindex[0]))
 
-        def available_cols(chr):
-            np.seterr(divide='ignore', invalid='ignore')
-            chr = str(chr)
-            chrsize = self.chromnames2sizes[chr]
-            itera = min(chrsize/self.resol/500, 25)
-            itera = np.uint64(itera)
-            unitsize = np.floor(chrsize/self.resol/itera)
-            unitsize = np.uint64(unitsize)
-            poolsum = 0
-            for it in range(itera):
-                test_region_start = np.uint64(unitsize * self.resol * it+1)
-                test_region_end = np.uint64(unitsize * self.resol * (it+1))
-                if test_region_start > test_region_end:
-                    a = test_region_start
-                    test_region_start = test_region_end
-                    test_region_end = a
-                position = str(chr) + ":" + str(test_region_start) + "-" + str(test_region_end)
-                mat = self.unbalLib.fetch(position, position)
-                mat = nantozero(mat)
-                mat = np.round(mat, 1)
-                matsum = np.sum(mat, axis=1)
-                zeroindex = np.where(matsum == 0)
-                poolsum += (len(matsum) - len(zeroindex[0]))
+                return poolsum
 
-            return poolsum
+            print('1. Calculating the number of available columns ... \n')
+            n_available_col = Parallel(n_jobs=self.core)(delayed(available_cols)(chr) for chr in tqdm(chromnames2))
 
-        print('1. Calculating the number of available columns ... \n')
-        n_available_col = Parallel(n_jobs=self.core)(delayed(available_cols)(chr) for chr in tqdm(chromnames2))
+            samplesize = (n_available_col/ np.sum(n_available_col)) * 1000
+            samplesize = np.uint64(samplesize)
+            dif = 1000 - np.sum(samplesize)
+            notzero = np.where(samplesize != 0)
+            chromnames2 = [chromnames2[i] for i in notzero[0]]
+            samplesize[0] += dif
 
-        samplesize = (n_available_col/ np.sum(n_available_col)) * 1000
-        samplesize = np.uint64(samplesize)
-        dif = 1000 - np.sum(samplesize)
-        notzero = np.where(samplesize != 0)
-        chromnames2 = [chromnames2[i] for i in notzero[0]]
-        samplesize[0] += dif
+            def main_null_calc(chr):
+                with np.errstate(divide='ignore', invalid='ignore'):
+                    background_size = 50000/self.resol
+                    background_up = np.floor(background_size / 2)
+                    background_down = background_size - background_up
+                    background_up = int(background_up)
+                    background_down = int(background_down)
+                    background_size = int(background_size)
+                    chr = str(chr)
+                    #c = np.where(chromnames2 == chr)[0]
+                    c = chromnames2.index(chr)
 
-        def main_null_calc(chr):
-            with np.errstate(divide='ignore', invalid='ignore'):
-                background_size = 50000/self.resol
-                background_up = np.floor(background_size / 2)
-                background_down = background_size - background_up
-                background_up = int(background_up)
-                background_down = int(background_down)
-                background_size = int(background_size)
-                chr = str(chr)
-                np.seterr(divide='ignore', invalid='ignore')
-                #c = np.where(chromnames2 == chr)[0]
-                c = chromnames2.index(chr)
+                    # Modified in Dec 11 2020
+                    ss = samplesize[c]
+                    chrsize = self.chromnames2sizes[chr]
+                    itera = min(chrsize/self.resol/500, 25)
+                    itera = int(itera)
+                    unitsize = np.floor(chrsize/self.resol/itera)
+                    unitsize = np.uint64(unitsize)
 
-                # Modified in Dec 11 2020
-                ss = samplesize[c]
-                chrsize = self.chromnames2sizes[chr]
-                itera = min(chrsize/self.resol/500, 25)
-                itera = np.uint64(itera)
-                unitsize = np.floor(chrsize/self.resol/itera)
-                unitsize = np.uint64(unitsize)
+                    bgleft = np.zeros((400, 0))
+                    bgright = np.zeros((400, 0))
 
-                bgleft = np.zeros((400, 0))
-                bgright = np.zeros((400, 0))
+                    n_pool = []
 
-                n_pool = []
+                    for it in range(itera):
+                        sss = int(ss/itera)
+                        test_region_start1 = int(unitsize * self.resol * it+1)
+                        test_region_start0 = test_region_start1 - (400*self.resol)
+                        test_region_end1 = int(unitsize * self.resol * (it+1))
+                        test_region_end2 = int(unitsize * self.resol * (it+1)+(400*self.resol))
+                        if test_region_end2 > chrsize:
+                            test_region_end2 = chrsize-1
+                        if test_region_end1 > chrsize - 400*self.resol:
+                            test_region_end1 = chrsize - 400*self.resol
+                        if test_region_start0 <= 1:
+                            test_region_start0 = 1
+                        test_region_start0 = int(test_region_start0)
 
-                for it in range(itera):
-                    sss = np.uint64(ss/itera)
-                    test_region_start1 = np.uint64(unitsize * self.resol * it+1)
-                    test_region_end1 = np.uint64(unitsize * self.resol * (it+1))
-                    test_region_start2 = np.uint64(unitsize * self.resol * it+1)
-                    test_region_end2 = np.uint64(unitsize * self.resol * (it+1)+(400*self.resol))
-                    if test_region_end2 > chrsize:
-                        test_region_end2 = chrsize-1
-                    if test_region_end1 > chrsize - 400*self.resol:
-                        test_region_end1 = chrsize - 400*self.resol
+                        position1 = str(chr) + ":" + str(test_region_start1) + "-" + str(test_region_end1)
+                        position2 = str(chr) + ":" + str(test_region_start0) + "-" + str(test_region_end2)
+                        mat = self.unbalLib.fetch(position1, position2)
+                        mat = nantozero(mat)
+                       # mat = np.round(mat, 1)
+                        nrow = mat.shape[0]
+                        matsum = np.sum(mat, axis=1)
+                        zeroindex = np.where(matsum == 0)
+                        pool = [x for x in list(range(nrow)) if x not in zeroindex[0].tolist()]
+                        pool = [x for x in pool if x > 20 and x < (unitsize - 20)]
+                        n_pool.append(len(pool))
+                        if len(pool) == 0:
+                            del mat
+                        elif len(pool) < sss:
+                            randval = random.choices(pool, k=len(pool))
+                            tableft = np.zeros((400, len(pool)))
+                            tabcenter = np.zeros((400, len(pool)))
+                            tabright = np.zeros((400, len(pool)))
 
-                    position1 = str(chr) + ":" + str(test_region_start1) + "-" + str(test_region_end1)
-                    position2 = str(chr) + ":" + str(test_region_start2) + "-" + str(test_region_end2)
-                    mat = self.unbalLib.fetch(position2, position1)
-                    mat = nantozero(mat)
-                    mat = np.round(mat, 1)
-                    nrow = mat.shape[0]
-                    matsum = np.sum(mat, axis=1)
-                    zeroindex = np.where(matsum == 0)
-                    pool = [x for x in list(range(nrow)) if x not in zeroindex[0].tolist()]
-                    n_pool.append(len(pool))
-                    if len(pool) == 0:
-                        del mat
-                    elif len(pool) < sss:
-                        randval = random.choices(pool, k=len(pool))
-                        tableft = np.zeros((400, len(pool)))
-                        tabcenter = np.zeros((400, len(pool)))
-                        tabright = np.zeros((400, len(pool)))
+                            for i in range(len(pool)):
+                                x = randval[i]
+                                for j in range(0,400):
+                                    det = np.random.choice([0,1])
+                                    if det == 0:
+                                        y = x + j
+                                    else:
+                                        y = x - j
+                                    if it > 0 :
+                                        y = y + 400
 
-                        for i in range(len(pool)):
-                            x = randval[i]
-                            for j in range(0,400):
-                                y = x + j
-                                tableft[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up-background_size):(x - background_up)])
-                                tabcenter[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up):(x + background_down)])
-                                tabright[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x + background_down):(x + background_down+background_size)])
-                        bgleft_temp = np.subtract(tabcenter, tableft)
-                        bgright_temp = np.subtract(tabcenter, tabright)
-                        bgleft = np.column_stack((bgleft, bgleft_temp))
-                        bgright = np.column_stack((bgright, bgright_temp))
-                        del mat
-                    else:
-                        randval = random.choices(pool, k=sss)
-                        tableft = np.zeros((400, sss))
-                        tabcenter = np.zeros((400, sss))
-                        tabright = np.zeros((400, sss))
 
-                        for i in range(sss):
+                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
+                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
+                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+
+                            bgleft_temp = np.subtract(tabcenter, tableft)
+                            bgright_temp = np.subtract(tabcenter, tabright)
+                            bgleft = np.column_stack((bgleft, bgleft_temp))
+                            bgright = np.column_stack((bgright, bgright_temp))
+                            del mat
+                        else:
+                            randval = random.choices(pool, k=sss)
+                            tableft = np.zeros((400, sss))
+                            tabcenter = np.zeros((400, sss))
+                            tabright = np.zeros((400, sss))
+
+                            for i in range(sss):
+                                x = randval[i]
+                                for j in range(0, 400):
+                                    det = np.random.choice([0,1])
+                                    if det == 0:
+                                        y = x + j
+                                    else:
+                                        y = x - j
+                                    if it > 0:
+                                        y = y + 400
+                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
+                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
+                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+
+                            bgleft_temp = np.subtract(tabcenter, tableft)
+                            bgright_temp = np.subtract(tabcenter, tabright)
+                            del mat
+                            bgleft = np.column_stack((bgleft, bgleft_temp))
+                            bgright = np.column_stack((bgright, bgright_temp))
+
+                    depl = int(ss - bgleft.shape[1])
+                    if depl > 0:
+                        rich = np.argmax(n_pool)
+                        test_region_start1 = int(unitsize * self.resol * rich+1)
+                        test_region_start0 = int(test_region_start1 - (400*self.resol))
+                        test_region_end1 = int(unitsize * self.resol * (rich+1))
+                        test_region_start2 = int(unitsize * self.resol * rich+1)
+                        test_region_end2 = int(unitsize * self.resol * (rich+1)+(400*self.resol))
+                        if test_region_end2 > chrsize:
+                            test_region_end2 = chrsize-1
+                        if test_region_end1 > chrsize - 400*self.resol:
+                            test_region_end1 = chrsize - 400*self.resol
+                        if test_region_start0 <= 1:
+                            test_region_start0 = 1
+
+                        position1 = str(chr) + ":" + str(test_region_start1) + "-" + str(test_region_end1)
+                        position2 = str(chr) + ":" + str(test_region_start0) + "-" + str(test_region_end2)
+                        mat = self.unbalLib.fetch(position1, position2)
+                        mat = nantozero(mat)
+                        #mat = np.round(mat, 1)
+                        nrow = mat.shape[0]
+                        matsum = np.sum(mat, axis=1)
+                        zeroindex = np.where(matsum == 0)
+                        pool = [x for x in list(range(nrow)) if x not in zeroindex[0].tolist()]
+                        pool = [x for x in pool if x > 20 and x < (unitsize - 20)]
+                        randval = random.choices(pool, k=depl)
+                        tableft = np.zeros((400, depl))
+                        tabcenter = np.zeros((400, depl))
+                        tabright = np.zeros((400, depl))
+
+                        for i in range(depl):
                             x = randval[i]
                             for j in range(0, 400):
-                                y = x + j
-                                tableft[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up - background_size):(x - background_up)])
-                                tabcenter[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up):(x + background_down)])
-                                tabright[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x + background_down):(x + background_down+background_size)])
+                                det = np.random.choice([0, 1])
+                                if det == 0:
+                                    y = x + j
+                                else:
+                                    y = x - j
+                                if it > 0:
+                                    y = y + 400
+                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
+                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
+                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+
                         bgleft_temp = np.subtract(tabcenter, tableft)
                         bgright_temp = np.subtract(tabcenter, tabright)
                         del mat
                         bgleft = np.column_stack((bgleft, bgleft_temp))
                         bgright = np.column_stack((bgright, bgright_temp))
 
-                depl = np.uint64(ss - bgleft.shape[1])
-                if depl > 0:
-                    rich = np.argmax(n_pool)
-                    test_region_start1 = np.uint64(unitsize * self.resol * rich+1)
-                    test_region_end1 = np.uint64(unitsize * self.resol * (rich+1))
-                    test_region_start2 = np.uint64(unitsize * self.resol * rich+1)
-                    test_region_end2 = np.uint64(unitsize * self.resol * (rich+1)+(400*self.resol))
-                    if test_region_end2 > chrsize:
-                        test_region_end2 = chrsize-1
-                    if test_region_end1 > chrsize - 400*self.resol:
-                        test_region_end1 = chrsize - 400*self.resol
+                        return bgleft, bgright
+            # apply parallel.
+            print('2. Constituting background ... \n')
+            result = Parallel(n_jobs=self.core)(delayed(main_null_calc)(chr) for chr in tqdm(chromnames2))
+            bgleft = np.zeros((400,0))
+            bgright = np.zeros((400,0))
+            for i in range(len(result)):
+                if(type(result[i]) == type(None)):
+                    continue
+                else:
+                    bl,br = result[i]
+                    bgleft=np.column_stack((bgleft,bl))
+                    bgright=np.column_stack((bgright,br))
 
-                    position1 = str(chr) + ":" + str(test_region_start1) + "-" + str(test_region_end1)
-                    position2 = str(chr) + ":" + str(test_region_start2) + "-" + str(test_region_end2)
-                    mat = self.unbalLib.fetch(position2, position1)
-                    mat = nantozero(mat)
-                    mat = np.round(mat, 1)
-                    nrow = mat.shape[0]
-                    matsum = np.sum(mat, axis=1)
-                    zeroindex = np.where(matsum == 0)
-                    pool = [x for x in list(range(nrow)) if x not in zeroindex[0].tolist()]
-                    randval = random.choices(pool, k=depl)
-                    tableft = np.zeros((400, depl))
-                    tabcenter = np.zeros((400, depl))
-                    tabright = np.zeros((400, depl))
-
-                    for i in range(depl):
-                        x = randval[i]
-                        for j in range(0, 400):
-                            y = x + j
-                            tableft[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up - background_size):(x - background_up)])
-                            tabcenter[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x - background_up):(x + background_down)])
-                            tabright[j, i] = np.mean(mat[(y - background_up):(y + background_down), (x + background_down):(x + background_down+background_size)])
-                    bgleft_temp = np.subtract(tabcenter, tableft)
-                    bgright_temp = np.subtract(tabcenter, tabright)
-                    del mat
-                    bgleft = np.column_stack((bgleft, bgleft_temp))
-                    bgright = np.column_stack((bgright, bgright_temp))
-
-                    return bgleft, bgright
-        # apply parallel.
-        print('2. Constituting background ... \n')
-        result = Parallel(n_jobs=self.core)(delayed(main_null_calc)(chr) for chr in tqdm(chromnames2))
-        bgleft = np.zeros((400,0))
-        bgright = np.zeros((400,0))
-        for i in range(len(result)):
-            if(type(result[i]) == type(None)):
-                continue
-            else:
-                bl,br = result[i]
-                bgleft=np.column_stack((bgleft,bl))
-                bgright=np.column_stack((bgright,br))
-
-        print('Elapsed time for background estimation: ' + str(np.round((time.time() - t_background_start) / 60, 3)) + ' min')
-        return bgleft, bgright
+            print('Elapsed time for background estimation: ' + str(np.round((time.time() - t_background_start) / 60, 3)) + ' min')
+            return bgleft, bgright
 
     def getMean(self, df, mask='0'):
 
@@ -440,14 +537,14 @@ class getStripe:
                 if x1 == y1:  # downward stripe
                     difference = j
                 elif x2 == y2:  # upward stripe
-                    difference = y2 - y1 - j
+                    difference = y2 - y1 - j - 1
                 if difference >= 400:
                     difference = 399
                 difference = int(difference)
                 bleft = bgleft[difference, :]
                 bright = bgright[difference, :]
-                p1 = len(np.where(bleft >= left_diff[j])[0]) / len(bleft)
-                p2 = len(np.where(bright >= right_diff[j])[0]) / len(bright)
+                p1 = len(np.where(bleft >= left_diff[j])[0]) / len(bleft[np.where(~np.isnan(bleft))])
+                p2 = len(np.where(bright >= right_diff[j])[0]) / len(bright[np.where(~np.isnan(bright))])
                 pval = max(p1, p2)
                 if pval == 0:
                     pval = 1/len(bleft)
@@ -596,10 +693,12 @@ class getStripe:
             xe = df['pos2'].iloc[i]
             ys = df['pos3'].iloc[i]
             ye = df['pos4'].iloc[i]
-            x_start_index = int((xs - 1) / self.resol)
+            #x_start_index = int((xs - 1) / self.resol)
+            x_start_index = int((xs) / self.resol) # to be deleted
             x_end_index = int(xe / self.resol)
 
-            y_start_index = int((ys - 1) / self.resol)
+            #y_start_index = int((ys - 1) / self.resol)
+            y_start_index = int((ys) / self.resol)# to be deleted
             y_end_index = int(ye / self.resol)
 
             leftmost = x_start_index - background_size
@@ -609,23 +708,23 @@ class getStripe:
             if rightmost > chrom_bin_size:
                 rightmost = chrom_bin_size-1
 
-            x_coord = str(c) + ":" + str(xs) + '-' + str(xe)
-            y_coord = str(c) + ":" + str(ys) + '-' + str(ye)
+            x_coord = str(c) + ":" + str(int(xs)) + '-' + str(int(xe))
+            y_coord = str(c) + ":" + str(int(ys)) + '-' + str(int(ye))
             center_obs = self.unbalLib.fetch(y_coord, x_coord)
             center_exp = expecMatrix(exval, x_start_index, x_end_index, y_start_index, y_end_index)
-            center_exp += 1
+            center_exp += .00000001
             center = np.divide(center_obs , center_exp)
 
             x_coord = str(c) + ":" + str(int(leftmost * self.resol)) + '-' + str(int(x_start_index * self.resol))
             left_obs = self.unbalLib.fetch(y_coord, x_coord)
             left_exp = expecMatrix(exval, leftmost, x_start_index, y_start_index, y_end_index)
-            left_exp += 1
+            left_exp += .00000001
             left = np.divide(left_obs, left_exp)
 
             x_coord = str(c) + ":" + str(int(x_end_index * self.resol)) + '-' + str(int(rightmost * self.resol))
             right_obs = self.unbalLib.fetch(y_coord,x_coord)
             right_exp = expecMatrix(exval, x_end_index + 1, rightmost +1, y_start_index, y_end_index)
-            right_exp += 1
+            right_exp += .00000001
             right = np.divide(right_obs, right_exp)
 
             if is_mask and mask_start > np.min(xs-50000, ys) and mask_start < np.max(xe+50000,ye):
@@ -636,21 +735,22 @@ class getStripe:
                 right = masking(right, mask_x_start, mask_x_end, x_end_index+1, rightmost, 1)
                 right = masking(right, mask_x_start, mask_x_end, y_start_index, y_end_index, 2)
 
+            centerm = np.mean(center, axis=1)
+            leftm = np.mean(left, axis=1)
+            rightm = np.mean(right, axis=1)
 
-            centerm = np.nanmean(center, axis=1)
-            leftm = np.nanmean(left, axis=1)
-            rightm = np.nanmean(right, axis=1)
-
-            centerTotal = np.nansum(center)
-            centerMean = np.nanmean(center)
+            centerTotal = np.sum(center[np.where(~np.isnan(center))[0]])
+            centerMean = np.mean(center[np.where(~np.isnan(center))[0]])
 
             g_xl = stats.elementwise_product_sum(K_xl, leftm, centerm)
             g_xr = stats.elementwise_product_sum(K_xr, centerm, rightm)
             g_y = stats.elementwise_product_sum(K_y, leftm, centerm, rightm)
             g_x = np.minimum(g_xl, g_xr)
             diff = [a - b for a, b in zip(g_x, g_y)]
-            avgdiff = np.nanmean(diff)
+            diff = [x for x in diff if x >= 0 or x < 0]
+            avgdiff = np.mean(diff)
             g = np.nanmedian(centerm) * avgdiff
+            g = float(g)
             return i,g,centerMean, centerTotal
 
         # Scoring
@@ -671,8 +771,9 @@ class getStripe:
             idx = np.where(df['chr'] == c)[0].tolist()
             chrom_idx = self.chromnames.index(c)
             chrom_bin_size = np.ceil(self.chromsizes[chrom_idx] / self.resol)
-
+            chrom_bin_size = int(chrom_bin_size)
             exval = expecVal[c]
+
             result = Parallel(n_jobs=self.core)(delayed(iterate_idx)(i,is_mask,exval) for i in tqdm(idx))
             for r in range(len(result)):
                 i,g,cM,cT = result[r]
@@ -731,7 +832,6 @@ class getStripe:
                 #del cfm
 
                 nframes = math.ceil(rowsize / 200)
-
                 results = Parallel(n_jobs = self.core)(delayed(search_frame)(i) for i in tqdm(range(nframes)))
 
                 for n in range(len(results)):
